@@ -9,15 +9,24 @@ isN0 <- function(x)  is.na(x) | x != 0
 is1  <- function(x) !is.na(x) & x   # also == "isTRUE componentwise"
 
 ##
-all0 <- function(x) !any(is.na(x)) && all(!x) ## ~= allFalse
-any0 <- function(x) isTRUE(any(x == 0))	      ## ~= anyFalse
+##allFalse <- function(x) !any(x) && !any(is.na(x))## ~= all0, but allFalse(NULL) = TRUE w/warning
+##all0 <- function(x) !any(is.na(x)) && all(!x) ## ~= allFalse
+allFalse <-
+all0 <- function(x) .Call(R_all0, x)
+
+##anyFalse <- function(x) isTRUE(any(!x))		 ## ~= any0
+## any0 <- function(x) isTRUE(any(x == 0))	      ## ~= anyFalse
+anyFalse <-
+any0 <- function(x) .Call(R_any0, x)
+
 ## These work "identically" for  1 ('==' TRUE)  and 0 ('==' FALSE)
 ##	(but give a warning for "double"  1 or 0)
 ## TODO: C versions of these would be faster
 allTrue  <- function(x) all(x)  && !any(is.na(x))
-allFalse <- function(x) !any(x) && !any(is.na(x))## ~= all0
-anyFalse <- function(x) isTRUE(any(!x))		 ## ~= any0
 
+
+## Note that mode(<integer>) = "numeric" -- as0(), as1() return "double"
+## which is good *AS LONG AS* we do not really have i..Matrix integer matrices
 as1 <- function(x, mod=mode(x))
     switch(mod, "integer"= 1L, "double"=, "numeric"= 1, "logical"= TRUE,
 	   "complex"= 1+0i, stop(gettextf("invalid 'mod': %s", mod), domain = NA))
@@ -75,6 +84,11 @@ Matrix.msg <- function(..., .M.level = 1) {
         message(...)
 }
 
+## TODO: faster via C, either R's  R_data_class() [which needs to become API !]
+##       or even direct  getAttrib(x, R_ClassSymbol); ..
+##' class - single string, no "package" attribute,..
+.class0 <- function(x)  as.vector(class(x))
+
 ## we can set this to FALSE and possibly measure speedup:
 .copyClass.check <- TRUE
 
@@ -136,24 +150,30 @@ attrSlotNames <- function(m, factors = TRUE) {
 attrSlots <- function(m, factors = TRUE) sapply(attrSlotNames(m, factors=factors),
 			 function(sn) slot(m, sn), simplify = FALSE)
 
+##' @return { NULL | TRUE | character | list(.) }
 attr.all_Mat <- function(target, current,
 			 check.attributes = TRUE, factorsCheck = FALSE, ...) {
     msg <- if(check.attributes)
 	all.equal(attrSlots(target,  factors=factorsCheck),
 		  attrSlots(current, factors=factorsCheck),
-		  check.attributes = TRUE, ...)
+		  check.attributes = TRUE, ...) ## else NULL
     if((c1 <- class(target)) != (c2 <- class(current)))
 	## list(): so we can easily check for this
-	list(c(msg, paste0("class(target) is ", c1, ", current is ", c2)))
+	list(c(if(!isTRUE(msg)) msg, paste0("class(target) is ", c1, ", current is ", c2)))
     else msg
 }
 
+##' @return combination for  all.equal() functions in ./Matrix.R & ./sparseMatrix.R
+.a.e.comb <- function(msg, r) {
+    if((is.null(msg) || isTRUE(msg)) & (r.ok <- isTRUE(r))) TRUE
+    else c(if(!isTRUE(msg)) msg, if(!r.ok) r)
+}
 
 
 ## chol() via "dpoMatrix"
 ## This will only be called for *dense* matrices
 cholMat <- function(x, pivot = FALSE, ...) {
-    packed <- length(x@x) < prod(dim(x)) ## is it packed?
+    packed <- .isPacked(x)
     nmCh <- if(packed) "pCholesky" else "Cholesky"
     if(!is.null(ch <- x@factors[[nmCh]]))
 	return(ch) ## use the cache
@@ -274,32 +294,84 @@ mmultCheck <- function(a, b, kind = 1L) {
     ca
 }
 
-dimNamesCheck <- function(a, b) {
-    ## Constructs "sensical" dimnames for something like  a + b ;
-    ## assume dimCheck() has happened before
+##' Constructs "sensical" dimnames for something like  a + b ;
+##' assume dimCheck() has happened before
+##'
+##' NOTA BENE:   R's  ?Arithmetic  says
+##' ---------
+##'>  For arrays (and an array result) the dimensions and dimnames are taken from
+##'>  first argument if it is an array, otherwise the second.
+##' but that's not quite correct:
+##' The dimnames are taken from second *if* the first are NULL.
+##'
+##' @title Construct dimnames for  a  o  b
+##' @param a matrix
+##' @param b matrix
+##' @param useFirst logical indicating if dimnames(a), the first, is taken, unless NULL
+##' @param check logical indicating if a warning should be signalled for mismatches
+##' @return a \code{\link{list}} of length two with dimnames
+##' @author Martin Maechler
+dimNamesCheck <- function(a, b, useFirst = TRUE, check = FALSE) {
     nullDN <- list(NULL,NULL)
     h.a <- !identical(nullDN, dna <- dimnames(a))
     h.b <- !identical(nullDN, dnb <- dimnames(b))
     if(h.a || h.b) {
-	if (!h.b) dna
-	else if(!h.a) dnb
-	else { ## both have non-trivial dimnames
-	    r <- dna # "default" result
-	    for(j in 1:2) if(!is.null(dn <- dnb[[j]])) {
-		if(is.null(r[[j]]))
-		    r[[j]] <- dn
-		else if(!identical(r[[j]], dn))
-		    warning(gettextf("dimnames [%d] mismatch in %s", j,
-				     deparse(sys.call(sys.parent()))),
-			    call. = FALSE, domain=NA)
-	    }
-	    r
-	}
+        if(useFirst) {
+            if(!h.a) dnb else dna
+        } else {
+            if (!h.b) dna
+            else if(!h.a) dnb
+            else { ## both have non-trivial dimnames
+                r <- dna # "default" result
+                for(j in 1:2) if(!is.null(dn <- dnb[[j]])) {
+                    if(is.null(r[[j]]))
+                        r[[j]] <- dn
+                    else if(check && !identical(r[[j]], dn))
+                        warning(gettextf("dimnames [%d] mismatch in %s", j,
+                                         deparse(sys.call(sys.parent()))),
+                                call. = FALSE, domain=NA)
+                }
+                r
+            }
+        }
     }
     else
 	nullDN
 }
 
+##' @title Symmetrize dimnames(.)
+##' @param x a square matrix
+##' @param col logical indicating if the column names should be taken when
+##' both are non-NULL.
+##' @param names logical indicating if the names(dimnames(.)) should be
+##' symmetrized and kept *if* they differ.
+##' @return a matrix like \code{x}, say \code{r}, with dimnames fulfilling
+##' 		dr <- dimnames(r); identical(dr[1], dr[2])
+##' @author Martin Maechler
+symmetrizeDimnames <- function(x, col=TRUE, names=TRUE) {
+    dimnames(x) <- symmDN(dimnames(x), col=col, names=names)
+    x
+}
+
+symmDN <- function(dn, col=TRUE, names=TRUE) {
+    if(is.null(dn) || identical(dn[1L], dn[2L]))
+	return(dn)
+    J <-
+        if(col) {
+            if(is.null(dn[[2L]])) 1L else 2L
+        } else { ## !col : row
+            if(is.null(dn[[1L]])) 2L else 1L
+        }
+
+    if(!is.null(n <- names(dn))) {
+	if(length(n) != 2)
+	    stop("names(dimnames(<matrix>)) must be NULL or of length two")
+	if(n[1L] != n[2L])
+	    names(dn) <- if(names) n[c(J,J)] # else NULL
+    }
+    dn[c(J,J)]
+}
+
 rowCheck <- function(a, b) {
     da <- dim(a)
     db <- dim(b)
@@ -361,10 +433,10 @@ isPacked <- function(x)
     ## Is 'x' a packed (dense) matrix ?
     is(x, "denseMatrix") &&
     ## unneeded(!): any("x" == slotNames(x)) &&
-    length(x@x) < prod(dim(x))
+    length(x@x) < prod(x@Dim)
 }
 ##" Is 'x' a packed (dense) matrix -- "no-check" version
-.isPacked <- function(x) length(x@x) < prod(dim(x))
+.isPacked <- function(x) length(x@x) < prod(x@Dim)
 
 emptyColnames <- function(x, msg.if.not.empty = FALSE)
 {
@@ -388,6 +460,7 @@ emptyColnames <- function(x, msg.if.not.empty = FALSE)
 }
 
 
+
 ## The i-th unit vector  e[1:n] with e[j] = \delta_{i,j}
 ## .E.i.log <- function(i,n)  i == (1:n)
 ## .E.i <- function(i,n)
@@ -575,48 +648,73 @@ if(FALSE) { ## -- now have  .Call(m_encodeInd, ...) etc :
 
 ## nr= nrow: since  i in {0,1,.., nrow-1}  these are 1L "decimal" encodings:
 ## Further, these map to and from the usual "Fortran-indexing" (but 0-based)
-## __ care against integer overflow __
 encodeInd  <- function(ij, di) {
     stopifnot(length(di) == 2)
     nr <- di[1L]
+    ## __ care against integer overflow __
     if(prod(di) >= .Machine$integer.max) nr <- as.double(nr)
     ij[,1] + ij[,2] * nr
 }
 encodeInd2 <- function(i,j, di) {
     stopifnot(length(di) == 2)
     nr <- di[1L]
+    ## __ care against integer overflow __
     if(prod(di) >= .Machine$integer.max) nr <- as.double(nr)
     i +  j * nr
 }
-}## no more needed
+} else {
+##' Encode Matrix index (i,j)  |-->  i + j * nrow   {i,j : 0-origin}
+##'
+##' @param ij 2-column integer matrix
+##' @param dim dim(.), i.e. length 2 integer vector
+##' @param checkBnds logical indicating  0 <= ij[,k] < di[k]  need to be checked.
+##'
+##' @return encoded index; integer if prod(dim) is small; double otherwise
+encodeInd <- function(ij, dim, orig1=FALSE, checkBnds=TRUE)
+    .Call(m_encodeInd, ij, dim, orig1, checkBnds)
+## --> in ../src/Mutils.c :  m_encodeInd(ij, di, orig_1, chk_bnds)
+##                           ~~~~~~~~~~~
 
-## 'code' is 0-based; as.integer(.) because encode*() may produce double
+##' Here, 1-based indices (i,j) are default:
+encodeInd2 <- function(i, j, dim, orig1=TRUE, checkBnds=TRUE)
+    .Call(m_encodeInd2, i,j, dim, orig1, checkBnds)
+
+}
+
+##' Decode "encoded" (i,j) indices back to  cbind(i,j)
+##' This is the inverse of encodeInd(.)
+##'
+##' @title Decode "Encoded" (i,j) Indices
+##' @param code integer in 0:((n x m - 1)  <==> encodeInd(.) result
+##' @param nr the number of rows
+##' @return
+##' @author Martin Maechler
 decodeInd <- function(code, nr) cbind(as.integer(code %% nr),
-				      as.integer(code %/% nr))
+				      as.integer(code %/% nr), deparse.level=0L)
 
-complementInd <- function(ij, dim, checkBnds=FALSE)
+complementInd <- function(ij, dim, orig1=FALSE, checkBnds=FALSE)
 {
     ## Purpose: Compute the complement of the 2-column 0-based ij-matrix
     ##		but as 1-based indices
     n <- prod(dim)
     if(n == 0) return(integer(0))
-    seq_len(n)[-(1L + .Call(m_encodeInd, ij, dim, checkBnds))]
+    seq_len(n)[-(1L + .Call(m_encodeInd, ij, dim, orig1, checkBnds))]
 }
 
 unionInd <- function(ij1, ij2) unique(rbind(ij1, ij2))
 
-intersectInd <- function(ij1, ij2, di, checkBnds=FALSE) {
+intersectInd <- function(ij1, ij2, di, orig1=FALSE, checkBnds=FALSE) {
     ## from 2-column (i,j) matrices where i in {0,.., nrow-1},
     ## return only the *common* entries
-    decodeInd(intersect(.Call(m_encodeInd, ij1, di, checkBnds),
-			.Call(m_encodeInd, ij2, di, checkBnds)), nr=di[1])
+    decodeInd(intersect(.Call(m_encodeInd, ij1, di, orig1, checkBnds),
+			.Call(m_encodeInd, ij2, di, orig1, checkBnds)), nr=di[1])
 }
 
-WhichintersectInd <- function(ij1, ij2, di, checkBnds=FALSE) {
+WhichintersectInd <- function(ij1, ij2, di, orig1=FALSE, checkBnds=FALSE) {
     ## from 2-column (i,j) matrices where i \in {0,.., nrow-1},
     ## find *where*  common entries are in ij1 & ij2
-    m1 <- match(.Call(m_encodeInd, ij1, di, checkBnds),
-                .Call(m_encodeInd, ij2, di, checkBnds))
+    m1 <- match(.Call(m_encodeInd, ij1, di, orig1, checkBnds),
+                .Call(m_encodeInd, ij2, di, orig1, checkBnds))
     ni <- !is.na(m1)
     list(which(ni), m1[ni])
 }
@@ -675,11 +773,11 @@ asTuniq <- function(x) {
 
 ## is 'x' a uniq Tsparse Matrix ?
 is_not_uniqT <- function(x, di = dim(x))
-    is.unsorted(x@j) || anyDuplicated(.Call(m_encodeInd2, x@i, x@j, di, FALSE))
+    is.unsorted(x@j) || anyDuplicated(.Call(m_encodeInd2, x@i, x@j, di, FALSE, FALSE))
 
 ## is 'x' a TsparseMatrix with duplicated entries (to be *added* for uniq):
 is_duplicatedT <- function(x, di = dim(x))
-    anyDuplicated(.Call(m_encodeInd2, x@i, x@j, di, FALSE))
+    anyDuplicated(.Call(m_encodeInd2, x@i, x@j, di, FALSE, FALSE))
 
 
 t_geMatrix <- function(x) {
@@ -779,10 +877,14 @@ tT2gT <- function(x, cl = class(x), toClass, cld = getClassDef(cl)) {
 ## Just *modify* the 'x' object , using attr(x, "class') <- toClass
 
 
-## Fast very special one
-## .gT2tC <- function(x, uplo, diag) .Call(Tsparse_to_tCsparse, x, uplo, diag)
+## Fast very special one ../src/Tsparse.c -- as_cholmod_triplet() in ../src/chm_common.c
+## 'x' *must* inherit from TsparseMatrix!
+.gT2tC <- function(x, uplo, diag="N") .Call(Tsparse_to_tCsparse, x, uplo, diag)
+## Ditto in ../src/Csparse.c :
+.gC2tC <- function(x, uplo, diag="N") .Call(Csparse_to_tCsparse, x, uplo, diag)
+.gC2tT <- function(x, uplo, diag="N") .Call(Csparse_to_tTsparse, x, uplo, diag)
 
-gT2tT <- function(x, uplo, diag, cl = class(x), toClass,
+gT2tT <- function(x, uplo, diag, toClass,
 		  do.n = extends(toClass, "nMatrix"))
 {
     ## coerce *gTMatrix to *tTMatrix {general -> triangular}
@@ -804,16 +906,35 @@ gT2tT <- function(x, uplo, diag, cl = class(x), toClass,
 	    x = x@x[sel], Dim = x@Dim, Dimnames = x@Dimnames)
 }
 
-check.gT2tT <- function(from, cl = MatrixClass(class(from)),
-			toClass, do.n = extends(toClass, "nMatrix")) {
+check.gT2tT <- function(from, toClass, do.n = extends(toClass, "nMatrix")) {
     if(isTr <- isTriangular(from)) {
-        force(cl)
 	gT2tT(from, uplo = .if.NULL(attr(isTr, "kind"), "U"),
 	      diag = "N", ## improve: also test for unit diagonal
-	      cl = cl, toClass = toClass, do.n = do.n)
+	      toClass = toClass, do.n = do.n)
     } else stop("not a triangular matrix")
 }
 
+gT2sT <- function(x, toClass, do.n = extends(toClass, "nMatrix")) {
+    upper <- x@i <= x@j
+    i <- x@i[upper]
+    j <- x@j[upper]
+    if(do.n) ## no 'x' slot
+	new("nsTMatrix", Dim = x@Dim, Dimnames = x@Dimnames,
+	    i = i, j = j, uplo = "U")
+    else
+	new(toClass, Dim = x@Dim, Dimnames = x@Dimnames,
+	    i = i, j = j, x = x@x[upper], uplo = "U")
+}
+
+check.gT2sT <- function(x, toClass, do.n = extends(toClass, "nMatrix"))
+{
+    if(isSymmetric(x))
+        gT2sT(x, toClass, do.n)
+    else
+	stop("not a symmetric matrix; consider forceSymmetric() or symmpart()")
+}
+
+
 if(FALSE)# unused
 l2d_meth <- function(x) {
     cl <- MatrixClass(class(x))
@@ -850,14 +971,23 @@ l2d_meth <- function(x) {
 .M.kindC <- function(clx) { ## 'clx': class() *or* classdefinition
     if(is.character(clx))		# < speedup: get it once
         clx <- getClassDef(clx)
-    if(extends(clx, "sparseVector")) ## shortcut
-	substr(as.character(clx@className), 1,1)
-    else if(extends(clx, "dMatrix")) "d"
-    else if(extends(clx, "nMatrix")) "n"
-    else if(extends(clx, "lMatrix")) "l"
-    else if(extends(clx, "indMatrix")) "n" # permutation -> pattern
-    else if(extends(clx, "zMatrix")) "z"
-    else if(extends(clx, "iMatrix")) "i"
+    ex <- extends(clx)
+    if(any(ex == "sparseVector")) {
+	## must work for class *extending* "dsparseVector" ==> cannot use  (clx@className) !
+	if     (any(ex == "dsparseVector")) "d"
+	else if(any(ex == "nsparseVector")) "n"
+	else if(any(ex == "lsparseVector")) "l"
+	else if(any(ex == "zsparseVector")) "z"
+	else if(any(ex == "isparseVector")) "i"
+	else stop(gettextf(" not yet implemented for %s", clx@className),
+		  domain = NA)
+    }
+    else if(any(ex == "dMatrix")) "d"
+    else if(any(ex == "nMatrix")) "n"
+    else if(any(ex == "lMatrix")) "l"
+    else if(any(ex == "indMatrix")) "n" # permutation -> pattern
+    else if(any(ex == "zMatrix")) "z"
+    else if(any(ex == "iMatrix")) "i"
     else stop(gettextf(" not yet implemented for %s", clx@className),
 	      domain = NA)
 }
@@ -869,7 +999,7 @@ l2d_meth <- function(x) {
 		"n" = "logical",
 		"z" = "complex")
 ## the reverse, a "version of" .M.kind(.):
-.kind.type <- names(.type.kind); names(.kind.type) <- as.vector(.type.kind)
+.kind.type <- setNames(names(.type.kind), as.vector(.type.kind))
 
 .M.shape <- function(x, clx = class(x)) {
     ## 'clx': class() *or* class definition of x
@@ -882,9 +1012,10 @@ l2d_meth <- function(x) {
     else {
 	if(is.character(clx)) # < speedup: get it once
 	    clx <- getClassDef(clx)
-	if(extends(clx, "diagonalMatrix"))  "d"
-	else if(extends(clx, "triangularMatrix"))"t"
-	else if(extends(clx, "symmetricMatrix")) "s"
+        ex <- extends(clx)
+	if(     any(ex == "diagonalMatrix"))  "d"
+	else if(any(ex == "triangularMatrix"))"t"
+	else if(any(ex == "symmetricMatrix")) "s"
 	else "g"
     }
 }
@@ -1048,6 +1179,20 @@ asTri <- function(from, newclass) {
     else stop("not a triangular matrix")
 }
 
+mat2tri <- function(from, sparse=NA) {
+    isTri <- isTriangular(from)
+    if(isTri) {
+	d <- dim(from)
+	if(is.na(sparse))
+	    sparse <- prod(d) > 2*sum(isN0(from)) ## <==> sparseDefault() in Matrix()
+	if(sparse)
+	    as(as(from, "sparseMatrix"), "triangularMatrix")
+	else
+	    new(paste0(.M.kind(from),"trMatrix"), x = base::as.vector(from),
+                Dim = d, Dimnames = .M.DN(from), uplo = attr(isTri, "kind"))
+    }
+    else stop("not a triangular matrix")
+}
 
 
 try_as <- function(x, classes, tryAnyway = FALSE) {
@@ -1149,16 +1294,20 @@ isTriC <- function(object, upper = NA) {
     }
 }
 
+
+## When the matrix is known to be [n x n] aka "square"
+## (need "vector-indexing" work for 'M'):
+.is.diagonal.sq.matrix <- function(M, n = dim(M)[1L])
+    all0(M[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
+
 .is.diagonal <- function(object) {
     ## "matrix" or "denseMatrix" (but not "diagonalMatrix")
     d <- dim(object)
-    if(d[1] != (n <- d[2])) FALSE
-    else if(is.matrix(object))
-        ## requires that "vector-indexing" works for 'object' :
-        all0(object[rep(c(FALSE, rep.int(TRUE,n)), length = n^2)])
+    if(d[1L] != (n <- d[2L])) FALSE
+    else if(is.matrix(object)) .is.diagonal.sq.matrix(object, n)
     else ## "denseMatrix" -- packed or unpacked
         if(is(object, "generalMatrix")) # "dge", "lge", ...
-            all0(object@x[rep(c(FALSE, rep.int(TRUE,n)), length = n^2)])
+	    .is.diagonal.sq.matrix(object@x, n)
         else { ## "dense" but not {diag, general}, i.e. triangular or symmetric:
             ## -> has 'uplo'  differentiate between packed and unpacked
 
@@ -1170,7 +1319,7 @@ isTriC <- function(object, upper = NA) {
             }
 
 ### very cheap workaround
-	    all0(as.matrix(object)[rep(c(FALSE, rep.int(TRUE,n)), length = n^2)])
+	    all0(as.matrix(object)[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
         }
 }
 
@@ -1275,7 +1424,8 @@ diagN2U <- function(x, cl = getClassDef(class(x)), checkDense = FALSE)
 	.dgC.0.factors(x)
 }
 
-.set.factors <- function(x, name, value) .Call(R_set_factors, x, value, name)
+.set.factors <- function(x, name, value, warn.no.slot=FALSE)
+    .Call(R_set_factors, x, value, name, warn.no.slot)
 
 ### Fast, much simplified version of tapply()
 tapply1 <- function (X, INDEX, FUN = NULL, ..., simplify = TRUE) {
@@ -1334,11 +1484,10 @@ all0Matrix <- function(n,m) {
 	p = rep.int(0L, m+1L))
 }
 
-setZero <- function(x) {
+.setZero <- function(x, newclass = if(d[1] == d[2]) "dsCMatrix" else "dgCMatrix") {
     ## all-0 matrix  from x  which must inherit from 'Matrix'
     d <- x@Dim
-    new(if(d[1] == d[2]) "dsCMatrix" else "dgCMatrix",
-	Dim = d, Dimnames = x@Dimnames, p = rep.int(0L, d[2]+1L))
+    new(newclass, Dim = d, Dimnames = x@Dimnames, p = rep.int(0L, d[2]+1L))
 }
 
 .M.vectorSub <- function(x,i) { ## e.g. M[0] , M[TRUE],  M[1:2]
@@ -1379,20 +1528,24 @@ setparts <- function(x,y, uniqueCheck = TRUE, check = TRUE) {
          int = if(n1 < n2) y[m1] else x[m2])
 }
 
-##' @title Warn about extraneous arguments in the "..."  (of its caller)
-## TODO NOTE:   R/src/library/base/R/seq.R  uses a simpler approach,
-##    *and* ngettext(.)  {->  singular/plural *and* translatable}
-##' @return
-##' @author Martin Maechler, June 2012
-chk.s <- function(...) {
-    if(length(list(...)))
-	warning(gettextf("arguments %s are disregarded in\n %s",
-			 sub(")$", '', sub("^list\\(", '', deparse(list(...), control=c()))),
-			 deparse(sys.call(-1), control=c())), call. = FALSE, domain=NA)
+##' @title Warn about extraneous arguments in the "..."  (of its caller).
+##' A merger of my approach and the one in seq.default()
+##' @author Martin Maechler, June 2012, May 2014
+##' @param ...
+##' @param which.call passed to sys.call().  A caller may use -2 if the message should
+##' mention *its* caller
+chk.s <- function(..., which.call = -1) {
+    if(nx <- length(list(...)))
+	warning(sprintf(ngettext(nx,
+                                 "extra argument %s will be disregarded in\n %s",
+                                 "extra arguments %s will be disregarded in\n %s"),
+                        sub(")$", '', sub("^list\\(", '', deparse(list(...), control=c()))),
+                        deparse(sys.call(which.call), control=c())),
+                call. = FALSE, domain=NA)
 }
 
 ##' *Only* to be used as function in
-##'    setMethod("Compare", ...., .Cmp.swap)  -->  ./Ops.R  & ./diagMatrix.R
+##'    setMethod.("Compare", ...., .Cmp.swap)  -->  ./Ops.R  & ./diagMatrix.R
 .Cmp.swap <- function(e1,e2) {
     ## "swap RHS and LHS" and use the method below:
     switch(.Generic,
