@@ -110,14 +110,13 @@ setMethod("isTriangular", signature(object = "matrix"), isTriMat)
 setMethod("isDiagonal", signature(object = "matrix"), .is.diagonal)
 
 ## The "catch all" methods -- far from optimal:
-setMethod("symmpart", signature(x = "Matrix"),
-	  function(x) as((x + t(x))/2, "symmetricMatrix"))
-setMethod("skewpart", signature(x = "Matrix"),
-	  function(x) (x - t(x))/2)
+setMethod("symmpart", signature(x = "Matrix"), function(x)
+    as(symmetrizeDimnames(x + t(x))/2, "symmetricMatrix"))
+setMethod("skewpart", signature(x = "Matrix"), function(x) symmetrizeDimnames(x - t(x))/2)
 
 ## FIXME: do this (similarly as for "ddense.." in C
-setMethod("symmpart", signature(x = "matrix"), function(x) (x + t(x))/2)
-setMethod("skewpart", signature(x = "matrix"), function(x) (x - t(x))/2)
+setMethod("symmpart", signature(x = "matrix"), function(x) symmetrizeDimnames(x + t(x))/2)
+setMethod("skewpart", signature(x = "matrix"), function(x) symmetrizeDimnames(x - t(x))/2)
 
 
 if(getRversion() >= "3.1.0")
@@ -143,8 +142,8 @@ dimnamesGets <- function (x, value) {
 	!(is.null(v2 <- value[[2]]) || length(v2) == d[2]))
 	stop(gettextf("invalid dimnames given for %s object", dQuote(class(x))),
 	     domain=NA)
-    x@Dimnames <- list(if(!is.null(v1)) as.character(v1),
-		       if(!is.null(v2)) as.character(v2))
+    x@Dimnames <- # preserve names(value)!
+	lapply(value, function(v) if(!is.null(v)) as.character(v))
     x
 }
 setMethod("dimnames<-", signature(x = "Matrix", value = "list"),
@@ -422,96 +421,6 @@ setMethod("image", "Matrix",
 
 ## Group Methods
 
-## --- "Summary" ------- have "ddense*" and "dsparse*" ones in ---> ./dMatrix.R <---
-##      -------          "diagMatrix" --> ./diagMatrix.R            ~~~~~~~~~~~
-## For all other Matrix objects {and note that  "all" and "any" have their own}:
-
-setMethod("all", signature(x = "Matrix"),
-	  function(x, ..., na.rm)
-	  callGeneric(as(x, "lMatrix"), ..., na.rm=na.rm))
-
-setMethod("any", signature(x = "Matrix"),
-	  function(x, ..., na.rm)
-	  callGeneric(as(x, "lMatrix"), ..., na.rm=na.rm))
-
-setMethod("Summary", signature(x = "Matrix", na.rm = "ANY"),
-	  function(x, ..., na.rm)
-	  callGeneric(as(x,"dMatrix"), ..., na.rm = na.rm))
-
-## Try to make   min(1, <Matrix>)  work, i.e., not dispatch on first arg to .Primitive
-## This for(..) gives {during installation}
-## Error in setGeneric(F, signature = "...") :
-##   ‘max’ is a primitive function;  methods can be defined, but the generic function is implicit, and cannot be changed.
-if(FALSE)
-for(F in c("max", "min", "range", "prod", "sum", "any", "all")) {
-    setGeneric(F, signature = "...")
-}
-### try on "min" for now --- ~/R/Pkgs/Rmpfr/R/mpfr.R is the example (for "pmin")
-if(FALSE)### This gives error message that the "ANY" is method is sealed ...
-setMethod("min", "ANY",
-	  function(..., na.rm = FALSE) {
-	      args <- list(...)
-	      if(all(isAtm <- vapply(args, is.atomic, NA)))
-		  return( base::min(..., na.rm = na.rm) )
-              ## else try to dispatch on an argument which is a Matrix.. or in a
-              if(any(isM <- vapply(args, is, NA, class2="Matrix"))) {
-                  ## swap the Matrix with the first argument
-                  i <- which.max(isM)# the first "Matrix"
-                  if(i == 1)
-                      stop("programming error: min() should have dispatched w/ 1st arg much earlier")
-              } else { ## if no "Matrix", take the first non-atomic argument
-                  ## (FIXME: should take the first for which there is a method !)
-                  i <- which.max(!isAtm)
-              }
-              ii <- seq_along(args)
-              ii[c(1,i)] <- c(i,1)
-              do.call(min, c(args[ii], list(na.rm=na.rm)))
-          })
-
-if(FALSE) { ## FIXME: it does *not* solve the problem anyway ..
-##
-##  (m <- Matrix(c(0,0,2:0), 3,5))
-##   min(1,m)
-##-> error, as it calls the .Primitive min() and that does *not* dispatch on 2nd arg
-##
-setMethod("Summary", signature(x = "ANY", na.rm = "ANY"),
-	  function(x, ..., na.rm) {
-          if(!length(a <- list(...))) (get(.Generic, envir=baseenv()))(x, na.rm=na.rm)
-          else {
-              if(!is.null(v <- getOption("Matrix.verbose")) && v >= 1)
-                  if(length(a) > 1)
-                      message(gettextf("in Summary(<ANY>, .): %s(<%s>, <%s>,...)\n",
-                                       .Generic, class(x), class(a[[1]])), domain = NA)
-                  else
-                      message(gettextf("in Summary(<ANY>, .): %s(<%s>, <%s>)\n",
-                                       .Generic, class(x), class(a[[1]])), domain = NA)
-
-              do.call(.Generic, c(x, a, list(na.rm=na.rm)))
-	  }})
-}## {does not help --> not used}
-
-Summary.l <- function(x, ..., na.rm) { ## must be method directly
-    if(.Generic %in% c("all", "any"))
-	callGeneric(x@x, ..., na.rm = na.rm)
-    else {
-	r <- callGeneric(as(x,"dMatrix"), ..., na.rm = na.rm)
-	if(!is.infinite(r) && .Generic != "prod") as.integer(r) else r
-    }
-}
-## almost identical:
-Summary.np <- function(x, ..., na.rm) {
-    if(.Generic %in% c("all", "any"))
-	callGeneric(as(x, "lMatrix"), ..., na.rm = na.rm)
-    else {
-	r <- callGeneric(as(x,"dMatrix"), ..., na.rm = na.rm)
-	if(!is.infinite(r) && .Generic != "prod") as.integer(r) else r
-    }
-}
-##
-setMethod("Summary", signature(x = "lMatrix", na.rm = "ANY"), Summary.l)
-setMethod("Summary", signature(x = "nMatrix", na.rm = "ANY"), Summary.np)
-setMethod("Summary", signature(x = "indMatrix", na.rm = "ANY"), Summary.np)
-
 ## NOTE:  "&" and "|"  are now in group "Logic" c "Ops" --> ./Ops.R
 ##        "!" is in ./not.R
 
