@@ -1,5 +1,7 @@
-#### ------- All "Ops"  group methods for all Matrix classes ------------
-###               ===
+####--- All "Ops" group methods for all Matrix classes (incl sparseVector) ------
+####         ===   but diagonalMatrix  -> ./diagMatrix.R and  abIndex.R
+####                                        ~~~~~~~~~~~~      ~~~~~~~~~
+
 ### Note that the "Ops" group consists of
 ### sub-groups   "Arith", "Compare", and "Logic"
 ###               -----    -------        -----
@@ -8,19 +10,13 @@
 ###       'Logic'   :=  '"&"', '"|"'  (( but *not* '"!"' since that has
 ###			                 only one argument ))
 
-## in shell, find them with
-##    egrep 'Method\("(Ops|Compare|Arith|Logic)"' R/*R
-
-### "Ops" ---- remember Ops = {Arith, Compare, Logic}  [Logic: since R 2.4.1]
-### -----
-
-
-### Note: diagonalMatrix are handled by special methods -> ./diagMatrix.R
-###       --------------                                   ~~~~~~~~~~~~~~
+## find them with M-x grep -E 'Method\("(Ops|Compare|Arith|Logic)"' *.R
+##                --------
 
 ### Design decision for *sparseMatrix*:
 ### work via Csparse  since Tsparse are not-unique (<-> slots not compatible)
 
+### Dimnames: (partly) via  dimNamesCheck()  [ ./Auxiliaries.R ]
 
 ### --  0 -- (not dense *or* sparse) -----------------------------------
 
@@ -240,7 +236,7 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
             ## Here, we assume that 'r' and the indices align (!)
             encI <- .Call(m_encodeInd,
                           non0ind(e1, cl1, uniqT=FALSE, xtendSymm=FALSE),
-                          di = d, checkBounds = FALSE)
+                          di = d, orig1=FALSE, checkBounds=FALSE)
             rx[1L + encI] <- r
             r <- new(lClass, x = rx, Dim = d, Dimnames = dimnames(e1))
         }
@@ -323,10 +319,13 @@ Ops.x.x <- function(e1, e2)
 	}
 	## now, in all cases @x should be matching & correct {only "uplo" part is used}
 	r <- callGeneric(e1@x, e2@x)
+	kr <- .M.kind(r)
+	if(kr == "d" && !is.double(r)) ## as "igeMatrix" does not yet exist!
+	    r <- as.double(r)
 	if(geM)
-	    new(paste0(.M.kind(r), "geMatrix"), x = r, Dim = d, Dimnames = dimnames(e1))
+	    new(paste0(kr, "geMatrix"), x = r, Dim = d, Dimnames = dimnames(e1))
 	else
-	    new(paste0(.M.kind(r), Mclass), x = r, Dim = d, Dimnames = dimnames(e1), uplo = e1@uplo)
+	    new(paste0(kr, Mclass), x = r, Dim = d, Dimnames = dimnames(e1), uplo = e1@uplo)
     }
     else {
 	r <- if(!dens1 && !dens2)
@@ -744,7 +743,7 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
             ## Here, we assume that 'r' and the indices align (!)
             encI <- .Call(m_encodeInd,
                           non0ind(e1, cl1, uniqT=FALSE, xtendSymm=FALSE),
-                          di = d, checkBounds = FALSE)
+                          di = d, orig1=FALSE, checkBounds=FALSE)
             rx[1L + encI] <- r
             r <- new(lClass, x = rx, Dim = d, Dimnames = dimnames(e1))
         }
@@ -970,13 +969,13 @@ setMethod("Arith", signature(e1 = "dsCMatrix", e2 = "dsCMatrix"),
 
 ##-------- originally from ./dgCMatrix.R --------------------
 
-.Arith.Csparse <- function(e1, e2, Generic, class., triangular = FALSE)
+.Arith.Csparse <- function(e1, e2, Generic, class., triangular = FALSE, check.dimnames = TRUE)
 {
     ## Generic is one of  "+", "-", "*", "^", "%%", "%/%", "/"
 
     ## triangular:  TRUE  iff e1,e2 are triangular  _and_  e1@uplo == e2@uplo
     d <- dimCheck(e1, e2)
-    dn <- dimNamesCheck(e1, e2)
+    dn <- dimNamesCheck(e1, e2, check = check.dimnames)
     if(triangular) {
 	## need these for the 'x' slots in any case
 	if (e1@diag == "U") e1 <- .Call(Csparse_diagU2N, e1)
@@ -1115,6 +1114,7 @@ setMethod("Arith", signature(e1 = "dtCMatrix", e2 = "dtCMatrix"),
 setMethod("Arith", signature(e1 = "CsparseMatrix", e2 = "numeric"), .Arith.CM.atom)
 setMethod("Arith", signature(e1 = "numeric", e2 = "CsparseMatrix"), .Arith.atom.CM)
 
+##' compute indices for recycling <numeric> of length 'len' to match sparseMatrix 'spM'
 .Ops.recycle.ind <- function(spM, len) {
     n <- prod(d <- dim(spM))
     if(n < len) stop("vector too long in Matrix - vector operation")
@@ -1122,7 +1122,7 @@ setMethod("Arith", signature(e1 = "numeric", e2 = "CsparseMatrix"), .Arith.atom.
         warning("longer object length\n\tis not a multiple of shorter object length")
     ## TODO(speedup!): construction of [1L + in0 %%len] via one .Call()
     in0 <- .Call(m_encodeInd, .Call(compressed_non_0_ij, spM, TRUE),
-                 d, FALSE)
+                 d, FALSE, FALSE)
     1L + in0 %% len
 }
 
