@@ -17,6 +17,10 @@ setAs("TsparseMatrix", "matrix",
 ## adjusted for triangular matrices not represented in cholmod
 .T.2.C <- function(from) .Call(Tsparse_to_Csparse, from, ##
 			       is(from, "triangularMatrix"))
+## fast, exported for power users
+.T2Cmat <- function(from, isTri = is(from, "triangularMatrix"))
+    .Call(Tsparse_to_Csparse, from, isTri)
+
 
 setAs("TsparseMatrix", "CsparseMatrix", .T.2.C)
 
@@ -335,10 +339,10 @@ replTmat <- function (x, i, j, ..., value)
 		       cl," to ",class(x))
 	}
 	nr <- di[1]
-	x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE)
+	x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE, FALSE)
 	if(anyDuplicated(x.i)) { ## == if(is_duplicatedT(x, di = di))
 	    x <- uniqTsparse(x)
-	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE)
+	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE, FALSE)
 	}
 
         n <- prod(di)
@@ -378,7 +382,7 @@ replTmat <- function (x, i, j, ..., value)
 		x@j <- x@j[sel]
 		if(has.x)
 		    x@x <- x@x[sel]
-		if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+		if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 		    x@factors <- list()
 	    }
 	    return(x)
@@ -426,7 +430,7 @@ replTmat <- function (x, i, j, ..., value)
 	}
 	x@i <- c(x@i, i %%  nr)
 	x@j <- c(x@j, i %/% nr)
-	if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+	if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	    x@factors <- list()
 	return(x)
     } ## {nargs = 3;  x[ii] <- value }
@@ -534,7 +538,7 @@ replTmat <- function (x, i, j, ..., value)
 	    x@j <- x@j[!sel]
             if(has.x)
 		x@x <- x@x[!sel]
-	    if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+	    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 		x@factors <- list()
 	}
 	return(x)
@@ -565,7 +569,7 @@ replTmat <- function (x, i, j, ..., value)
             if(has.x)
                 x@x <- c(x@x, value)
         }
-	if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+	if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	    x@factors <- list()
         return(x)
     }
@@ -612,9 +616,10 @@ replTmat <- function (x, i, j, ..., value)
 
     if(any(sel)) {
 	## the 0-based indices of non-zero entries -- WRT to submatrix
-	non0 <- cbind(match(x@i[sel], i1),
-		      match(x@j[sel], i2)) - 1L
-	iN0 <- 1L + .Call(m_encodeInd, non0, di = dind, FALSE)
+	iN0 <- 1L + .Call(m_encodeInd2,
+			  match(x@i[sel], i1),
+			  match(x@j[sel], i2),
+			  di = dind, orig1=TRUE, FALSE)
 
 	## 1a) replace those that are already non-zero with non-0 values
 	vN0 <- isN0(value[iN0])
@@ -661,7 +666,7 @@ replTmat <- function (x, i, j, ..., value)
 		x@x <- c(x@x, as.vector(value[iIN0]))
 	}
     }
-    if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	x@factors <- list()
     x
 } ## end{replTmat}
@@ -778,7 +783,7 @@ replTmat <- function (x, i, j, ..., value)
 	clDx <- getClassDef(clx <- class(x))
     }
 
-    ii.v <- .Call(m_encodeInd, i - 1L, di, checkBounds = TRUE)# 0-indexing
+    ii.v <- .Call(m_encodeInd, i, di, orig1=TRUE, checkBounds = TRUE)
     if(id <- anyDuplicated(ii.v, fromLast=TRUE)) {
         Matrix.msg("duplicate ij-entries in 'Matrix[ ij ] <- value'; using last",
                    .M.level = 1)
@@ -790,7 +795,7 @@ replTmat <- function (x, i, j, ..., value)
             value <- value[nd]
         }
     }
-    ii.x <- .Call(m_encodeInd2, x@i, x@j, di, FALSE)
+    ii.x <- .Call(m_encodeInd2, x@i, x@j, di, FALSE, FALSE)
     m1 <- match(ii.v, ii.x)
     i.repl <- !is.na(m1) # those that need to be *replaced*
 
@@ -821,7 +826,7 @@ replTmat <- function (x, i, j, ..., value)
 	    x@x <- c(x@x, value[i.new])
     }
 
-    if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	x@factors <- list()
     x
 } ## end{.TM.repl.i.mat}
