@@ -30,6 +30,12 @@
     dim(x) <- if(ncol(y) == (n <- length(x))) c(1L, n) else c(n, 1L)
     tcrossprod(x, y)
 }
+## tcrossprod(<Mat>, <sparseVector>)
+.M.vt <- function(x, y) tcrossprod(x,
+				   if(nrow(x) == 1L)
+				       spV2M(y, nrow=1L, ncol=y@length, check=FALSE)
+				   else
+				       spV2M(y, nrow=y@length, ncol=1L, check=FALSE))
 
 ###-- I --- %*% ------------------------------------------------------
 
@@ -148,7 +154,7 @@ setMethod("%*%", signature(x = "CsparseMatrix", y = "ddenseMatrix"),
 	  function(x, y) .Call(Csparse_dense_prod, x, y))
 setMethod("%*%", signature(x = "CsparseMatrix", y = "matrix"),
 	  function(x, y) .Call(Csparse_dense_prod, x, y)) # was  x %*% Matrix(y)
-setMethod("%*%", signature(x = "CsparseMatrix", y = "numeric"),
+setMethod("%*%", signature(x = "CsparseMatrix", y = "numLike"),
 	  function(x, y) .Call(Csparse_dense_prod, x, y))
 
 
@@ -217,9 +223,22 @@ setMethod("%*%", signature(x = "Matrix", y = "ANY"), .local.bail.out)
 ### sparseVector
 sp.x.sp <- function(x, y) Matrix(sum(x * y), 1L, 1L, sparse=FALSE)
     ## inner product -- no sense to return sparse!
-setMethod("%*%", signature(x = "Matrix", y = "sparseVector"), .M.v)
-setMethod("%*%", signature(x = "sparseVector", y = "Matrix"), .v.M)
-setMethod("%*%", signature(x = "sparseVector", y = "sparseVector"), sp.x.sp)
+sp.X.sp <- function(x, y) {
+    if((n <- length(x)) == length(y)) sp.x.sp(x,y)
+    else if(n == 1L) spV2M(x, nrow = 1L, ncol = 1L, check = FALSE) %*% y
+    else stop("non-conformable arguments")
+}
+v.X.sp <- function(x, y) {
+    if((n <- length(x)) == length(y)) sp.x.sp(x,y)
+    else if(n == 1L) matrix(x, nrow = 1L, ncol = 1L) %*% y
+    else stop("non-conformable arguments")
+}
+
+setMethod("%*%", signature(x = "mMatrix", y = "sparseVector"), .M.v)
+setMethod("%*%", signature(x = "sparseVector", y = "mMatrix"), .v.M)
+setMethod("%*%", signature(x = "sparseVector", y = "sparseVector"), sp.X.sp)
+setMethod("%*%", signature(x = "sparseVector", y = "numLike"),      sp.X.sp)
+setMethod("%*%", signature(x = "numLike",      y = "sparseVector"), v.X.sp)
 ## setMethod("%*%", signature(x = "sparseMatrix", y = "sparseVector"),
 ##           function(x, y) x %*% .sparseV2Mat(y))
 
@@ -237,7 +256,7 @@ setMethod("crossprod", signature(x = "dgeMatrix", y = "dgeMatrix"),
 setMethod("crossprod", signature(x = "dgeMatrix", y = "matrix"),
 	  function(x, y = NULL) .Call(dgeMatrix_matrix_crossprod, x, y, FALSE),
 	  valueClass = "dgeMatrix")
-setMethod("crossprod", signature(x = "dgeMatrix", y = "numeric"),
+setMethod("crossprod", signature(x = "dgeMatrix", y = "numLike"),
 	  function(x, y = NULL) .Call(dgeMatrix_matrix_crossprod, x, y, FALSE),
 	  valueClass = "dgeMatrix")
 setMethod("crossprod", signature(x = "matrix", y = "dgeMatrix"),
@@ -310,13 +329,12 @@ setMethod("crossprod", signature(x = "CsparseMatrix", y = "CsparseMatrix"),
 	  function(x, y = NULL)
 	  .Call(Csparse_Csparse_crossprod, x, y, trans = FALSE))
 
-## FIXME: Generalize the class of y.  This specific method is to replace one
-##	  in dgCMatrix.R
+## FIXME: Generalize the class of y. (?? still ??)
 setMethod("crossprod", signature(x = "CsparseMatrix", y = "ddenseMatrix"),
 	  function(x, y = NULL) .Call(Csparse_dense_crossprod, x, y))
 setMethod("crossprod", signature(x = "CsparseMatrix", y = "matrix"),
 	  function(x, y = NULL) .Call(Csparse_dense_crossprod, x, y))
-setMethod("crossprod", signature(x = "CsparseMatrix", y = "numeric"),
+setMethod("crossprod", signature(x = "CsparseMatrix", y = "numLike"),
 	  function(x, y = NULL) .Call(Csparse_dense_crossprod, x, y))
 
 
@@ -391,15 +409,15 @@ setMethod("crossprod", signature(x = "ddenseMatrix", y = "CsparseMatrix"),
 	  function(x, y) t(.Call(Csparse_dense_crossprod, y, x)))
 setMethod("crossprod", signature(x = "matrix",	     y = "CsparseMatrix"),
 	  function(x, y) t(.Call(Csparse_dense_crossprod, y, x)))
-setMethod("crossprod", signature(x = "numeric",	     y = "CsparseMatrix"),
+setMethod("crossprod", signature(x = "numLike",	     y = "CsparseMatrix"),
 	  function(x, y) t(.Call(Csparse_dense_crossprod, y, x)))
 
 
 ## "Matrix" : cbind(), rbind() do  names -> dimnames
 setMethod("crossprod", signature(x = "Matrix", y = "numLike"),
-	  function(x, y) crossprod(x, cbind(y,deparse.level=0)))
-setMethod("crossprod", signature(x = "numLike", y = "Matrix"), 
-	  function(x, y) crossprod(rbind(x,deparse.level=0), y))
+	  function(x, y) crossprod(x, cbind(y, deparse.level=0)))
+setMethod("crossprod", signature(x = "numLike", y = "Matrix"),
+	  function(x, y) crossprod(cbind(x, deparse.level=0), y))
 
 setMethod("crossprod", signature(x = "Matrix", y = "matrix"),
 	  function(x, y) crossprod(x, Matrix(y)))
@@ -407,22 +425,46 @@ setMethod("crossprod", signature(x = "matrix", y = "Matrix"),
 	  function(x, y) crossprod(Matrix(x), y))
 
 ## sparseVector
-setMethod("crossprod", signature(x = "Matrix", y = "sparseVector"),
-	  function(x, y) crossprod(x, .sparseV2Mat(y)))
-setMethod("crossprod", signature(x = "sparseVector", y = "Matrix"), 
+setMethod("crossprod", signature(x = "mMatrix", y = "sparseVector"),
+	  function(x, y) crossprod(x,
+				   if(nrow(x) == 1L)
+				       spV2M(y, nrow=1L, ncol=y@length, check=FALSE)
+				   else
+				       spV2M(y, nrow=y@length, ncol=1L, check=FALSE)))
+
+setMethod("crossprod", signature(x = "sparseVector", y = "mMatrix"),
 	  function(x, y)
 	  crossprod(spV2M(x, nrow = length(x), ncol = 1L, check = FALSE), y))
-setMethod("crossprod", signature(x = "sparseVector", y = "sparseVector"), sp.x.sp)
+setMethod("crossprod", signature(x = "sparseVector", y = "sparseVector"), sp.X.sp)
+setMethod("crossprod", signature(x = "sparseVector", y = "numLike"),      sp.X.sp)
+setMethod("crossprod", signature(x = "numLike",      y = "sparseVector"), v.X.sp)
 setMethod("crossprod", signature(x = "sparseVector", y = "missing"),
 	  function(x, y=NULL) sp.x.sp(x,x))
 
+## Fallbacks -- symmetric LHS --> saving a t(.):
+##  {FIXME: want the method to be `%*%` -- but primitives are not allowed as methods}
+setMethod("crossprod", signature(x = "symmetricMatrix", y = "Matrix"),	function(x,y) x %*% y)
+setMethod("crossprod", signature(x = "symmetricMatrix", y = "missing"), function(x,y) x %*% x)
+setMethod("crossprod", signature(x = "symmetricMatrix", y = "ANY"),	function(x,y) x %*% y)
+##
 ## cheap fallbacks
 setMethod("crossprod", signature(x = "Matrix", y = "Matrix"),
-	  function(x, y) t(x) %*% y)
+	  function(x, y) {
+	      Matrix.msg(sprintf(
+	  "potentially suboptimal crossprod(\"%s\",\"%s\") as t(.) %s y",
+		  class(x), class(y), "%*%"))
+	      t(x) %*% y })
 setMethod("crossprod", signature(x = "Matrix", y = "missing"),
-	  function(x, y) t(x) %*% x)
+	  function(x, y) {
+	      Matrix.msg(paste0(
+	  "potentially suboptimal crossprod(<",class(x),">) as t(.) %*% . "))
+	      t(x) %*% x })
 setMethod("crossprod", signature(x = "Matrix", y = "ANY"),
-	  function(x, y) t(x) %*% y)
+	  function(x, y) {
+	      Matrix.msg(sprintf(
+	  "potentially suboptimal crossprod(\"%s\", <%s>[=<ANY>]) as t(.) %s y",
+		  class(x), class(y), "%*%"))
+	      t(x) %*% y })
 setMethod("crossprod", signature(x = "ANY", y = "Matrix"),
 	  function(x, y) t(x) %*% y)
 
@@ -512,12 +554,24 @@ setMethod("tcrossprod", signature(x = "CsparseMatrix", y = "ddenseMatrix"),
 setMethod("tcrossprod", signature(x = "CsparseMatrix", y = "matrix"),
 	  function(x, y) .Call(Csparse_dense_prod, x, t(y)))
 setMethod("tcrossprod", signature(x = "CsparseMatrix", y = "numLike"),
-	  function(x, y) .Call(Csparse_dense_prod, x, rbind(y, deparse.level=0)))
+	  function(x, y) .Call(Csparse_dense_prod, x,
+			       if(nrow(x) == 1L)
+				   cbind(y, deparse.level=0)
+			       else
+				   rbind(y, deparse.level=0)))
 
 
+### -- xy' = (yx')' --------------------
 ### FIXME (speed):  Csparse_dense_crossprod  should also get a 'trans = TRUE'
 ## so we could have one less t(<dense>) in this:
-## -- xy' = (yx')'
+for(T in c("d", "l", "n")) { ## speedup for *symmetric* RHS
+    .sCMatrix <- paste0(T, "sCMatrix")
+    setMethod("tcrossprod", signature(x = "ddenseMatrix", y = .sCMatrix),
+	      function(x, y) t(.Call(Csparse_dense_prod, y, x)))
+    setMethod("tcrossprod", signature(x = "matrix", y = .sCMatrix),
+	      function(x, y) t(.Call(Csparse_dense_prod, y, x)))
+}
+rm(T, .sCMatrix)
 setMethod("tcrossprod", signature(x = "ddenseMatrix", y = "CsparseMatrix"),
 	  function(x, y) t(.Call(Csparse_dense_prod, y, t(x))))
 setMethod("tcrossprod", signature(x = "matrix",	      y = "CsparseMatrix"),
@@ -550,8 +604,8 @@ setMethod("tcrossprod", signature(x = "TsparseMatrix", y = "TsparseMatrix"),
 
 
 ## "Matrix"
-setMethod("tcrossprod", signature(x = "Matrix", y = "numLike"), 
-	  function(x, y) x %*% rbind(y,deparse.level=0))
+setMethod("tcrossprod", signature(x = "Matrix", y = "numLike"),
+	  function(x, y) x %*% rbind(y, deparse.level=0))
 setMethod("tcrossprod", signature(x = "numLike", y = "Matrix"), .v.Mt)
 setMethod("tcrossprod", signature(x = "Matrix", y = "matrix"),
 	  function(x, y = NULL) tcrossprod(x, Matrix(y)))
@@ -559,12 +613,11 @@ setMethod("tcrossprod", signature(x = "matrix", y = "Matrix"),
 	  function(x, y = NULL) tcrossprod(Matrix(x), y))
 
 ## sparseVector
-setMethod("tcrossprod", signature(x = "Matrix", y = "sparseVector"), 
-	  function(x, y) tcrossprod(x, .sparseV2Mat(y)))
-setMethod("tcrossprod", signature(x = "sparseVector", y = "Matrix"), .v.Mt)
-setMethod("tcrossprod", signature(x = "sparseMatrix", y = "sparseVector"),
-	  function(x, y) tcrossprod(x, .sparseV2Mat(y)))
+## NB: the two "sparseMatrix" are "unneeded", only used to avoid ambiguity warning
+setMethod("tcrossprod", signature(x = "sparseMatrix", y = "sparseVector"), .M.vt)
+setMethod("tcrossprod", signature(x = "mMatrix",      y = "sparseVector"), .M.vt)
 setMethod("tcrossprod", signature(x = "sparseVector", y = "sparseMatrix"), .v.Mt)
+setMethod("tcrossprod", signature(x = "sparseVector", y = "mMatrix"),	   .v.Mt)
 setMethod("tcrossprod", signature(x = "sparseVector", y = "sparseVector"),
 	  function(x, y) .sparseV2Mat(x) %*%
           spV2M(y, nrow=1L, ncol=length(y), check=FALSE))
@@ -572,17 +625,37 @@ setMethod("tcrossprod", signature(x = "sparseVector", y = "missing"),
 	  ## could be speeded: spV2M(x, *) called twice with different ncol/nrow
 	  function(x, y=NULL) .sparseV2Mat(x) %*%
 	  spV2M(x, nrow=1L, ncol=length(x), check=FALSE))
+setMethod("tcrossprod", signature(x = "numLike",      y = "sparseVector"),
+	  function(x, y) tcrossprod(x, .sparseV2Mat(y)))
+setMethod("tcrossprod", signature(x = "sparseVector", y = "numLike"),
+	  function(x, y) .sparseV2Mat(x) %*% t(x))
 
 
+## Fallbacks -- symmetric RHS --> saving a t(.):
+##  {FIXME: want the method to be `%*%` -- but primitives are not allowed as methods}
+setMethod("tcrossprod", signature(x = "Matrix", y = "symmetricMatrix"),	function(x,y) x %*% y)
+setMethod("tcrossprod", signature(x = "ANY",    y = "symmetricMatrix"),	function(x,y) x %*% y)
+##
 ## cheap fallbacks
 setMethod("tcrossprod", signature(x = "Matrix", y = "Matrix"),
-	  function(x, y = NULL) x %*% t(y))
+	  function(x, y = NULL) {
+	      Matrix.msg(sprintf(
+	  "potentially suboptimal tcrossprod(\"%s\",\"%s\") as  x %s t(y)",
+		  class(x), class(y), "%*%"))
+	      x %*% t(y) })
 setMethod("tcrossprod", signature(x = "Matrix", y = "missing"),
-	  function(x, y = NULL) x %*% t(x))
+	  function(x, y = NULL) {
+	      Matrix.msg(paste0(
+	  "potentially suboptimal crossprod(<",class(x), ">) as  . %*% t(.)"))
+	      x %*% t(x) })
 setMethod("tcrossprod", signature(x = "Matrix", y = "ANY"),
 	  function(x, y = NULL) x %*% t(y))
 setMethod("tcrossprod", signature(x = "ANY", y = "Matrix"),
-	  function(x, y = NULL) x %*% t(y))
+	  function(x, y = NULL) {
+	      Matrix.msg(sprintf(
+	  "potentially suboptimal tcrossprod(<%s>[=<ANY>], \"%s\") as  x %s t(y)",
+		  class(x), class(y), "%*%"))
+	      x %*% t(y) })
 
 ## Local variables:
 ## mode: R
