@@ -186,9 +186,9 @@ SEXP nz2Csparse(SEXP x, enum x_slot_kind r_kind)
     return ans;
 }
 
-SEXP Csparse_to_matrix(SEXP x)
+SEXP Csparse_to_matrix(SEXP x, SEXP chk)
 {
-    return chm_dense_to_matrix(cholmod_sparse_to_dense(AS_CHM_SP__(x), &c),
+    return chm_dense_to_matrix(cholmod_sparse_to_dense(AS_CHM_SP2(x, asLogical(chk)), &c),
 			       1 /*do_free*/, GET_SLOT(x, Matrix_DimNamesSym));
 }
 SEXP Csparse_to_vector(SEXP x)
@@ -210,6 +210,30 @@ SEXP Csparse_to_Tsparse(SEXP x, SEXP tri)
 			       GET_SLOT(x, Matrix_DimNamesSym));
 }
 
+SEXP Csparse_to_tCsparse(SEXP x, SEXP uplo, SEXP diag)
+{
+    CHM_SP chxs = AS_CHM_SP__(x);
+    int Rkind = (chxs->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : 0;
+    R_CheckStack();
+    return chm_sparse_to_SEXP(chxs, /* dofree = */ 0,
+			      /* uploT = */ (*CHAR(asChar(uplo)) == 'U')? 1: -1,
+			       Rkind, /* diag = */ CHAR(STRING_ELT(diag, 0)),
+			       GET_SLOT(x, Matrix_DimNamesSym));
+}
+
+SEXP Csparse_to_tTsparse(SEXP x, SEXP uplo, SEXP diag)
+{
+    CHM_SP chxs = AS_CHM_SP__(x);
+    CHM_TR chxt = cholmod_sparse_to_triplet(chxs, &c);
+    int Rkind = (chxs->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : 0;
+    R_CheckStack();
+    return chm_triplet_to_SEXP(chxt, 1,
+			      /* uploT = */ (*CHAR(asChar(uplo)) == 'U')? 1: -1,
+			       Rkind, /* diag = */ CHAR(STRING_ELT(diag, 0)),
+			       GET_SLOT(x, Matrix_DimNamesSym));
+}
+
+
 /* this used to be called  sCMatrix_to_gCMatrix(..)   [in ./dsCMatrix.c ]: */
 SEXP Csparse_symmetric_to_general(SEXP x)
 {
@@ -233,13 +257,34 @@ SEXP Csparse_general_to_symmetric(SEXP x, SEXP uplo)
 	return R_NilValue; /* -Wall */
     }
     CHM_SP chx = AS_CHM_SP__(x), chgx;
-    int uploT = (*CHAR(STRING_ELT(uplo,0)) == 'U') ? 1 : -1;
+    int uploT = (*CHAR(asChar(uplo)) == 'U') ? 1 : -1;
     int Rkind = (chx->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : 0;
     R_CheckStack();
     chgx = cholmod_copy(chx, /* stype: */ uploT, chx->xtype, &c);
+
+    /* need _symmetric_ dimnames */
+    SEXP dns = PROTECT(duplicate(GET_SLOT(x, Matrix_DimNamesSym))),
+	nms_dns = getAttrib(dns, R_NamesSymbol);
+    if(!equal_string_vectors(VECTOR_ELT(dns, 0),
+			     VECTOR_ELT(dns, 1))) {
+	if(uploT == 1)
+	    SET_VECTOR_ELT(dns, 0, VECTOR_ELT(dns,1));
+	else
+	    SET_VECTOR_ELT(dns, 1, VECTOR_ELT(dns,0));
+    }
+    if(!isNull(nms_dns) &&  // names(dimnames(.)) :
+       !R_compute_identical(STRING_ELT(nms_dns, 0),
+			    STRING_ELT(nms_dns, 1), 15)) {
+	if(uploT == 1)
+	    SET_STRING_ELT(nms_dns, 0, STRING_ELT(nms_dns,1));
+	else
+	    SET_STRING_ELT(nms_dns, 1, STRING_ELT(nms_dns,0));
+	setAttrib(dns, R_NamesSymbol, nms_dns);
+    }
+
+    UNPROTECT(1);
     /* xtype: pattern, "real", complex or .. */
-    return chm_sparse_to_SEXP(chgx, 1, 0, Rkind, "",
-			      GET_SLOT(x, Matrix_DimNamesSym));
+    return chm_sparse_to_SEXP(chgx, 1, 0, Rkind, "", dns);
 }
 
 SEXP Csparse_transpose(SEXP x, SEXP tri)
@@ -256,6 +301,13 @@ SEXP Csparse_transpose(SEXP x, SEXP tri)
     tmp = VECTOR_ELT(dn, 0);	/* swap the dimnames */
     SET_VECTOR_ELT(dn, 0, VECTOR_ELT(dn, 1));
     SET_VECTOR_ELT(dn, 1, tmp);
+    if(!isNull(tmp = getAttrib(dn, R_NamesSymbol))) { // swap names(dimnames(.)):
+	SEXP nms_dns = PROTECT(allocVector(VECSXP, 2));
+	SET_VECTOR_ELT(nms_dns, 1, STRING_ELT(tmp, 0));
+        SET_VECTOR_ELT(nms_dns, 0, STRING_ELT(tmp, 1));
+	setAttrib(dn, R_NamesSymbol, nms_dns);
+	UNPROTECT(1);
+    }
     UNPROTECT(1);
     return chm_sparse_to_SEXP(chxt, 1, /* SWAP 'uplo' for triangular */
 			      tr ? ((*uplo_P(x) == 'U') ? -1 : 1) : 0,
@@ -345,7 +397,8 @@ SEXP Csparse_Csparse_crossprod(SEXP a, SEXP b, SEXP trans)
 SEXP Csparse_dense_prod(SEXP a, SEXP b)
 {
     CHM_SP cha = AS_CHM_SP(a);
-    SEXP b_M = PROTECT(mMatrix_as_dgeMatrix(b));
+    SEXP b_M = PROTECT(mMatrix_as_dgeMatrix2(b, // transpose_if_vector =
+					     cha->ncol == 1));
     CHM_DN chb = AS_CHM_DN(b_M);
     CHM_DN chc = cholmod_allocate_dense(cha->nrow, chb->ncol, cha->nrow,
 					chb->xtype, &c);
@@ -376,7 +429,8 @@ SEXP Csparse_dense_prod(SEXP a, SEXP b)
 SEXP Csparse_dense_crossprod(SEXP a, SEXP b)
 {
     CHM_SP cha = AS_CHM_SP(a);
-    SEXP b_M = PROTECT(mMatrix_as_dgeMatrix(b));
+    SEXP b_M = PROTECT(mMatrix_as_dgeMatrix2(b, // transpose_if_vector =
+					     cha->nrow == 1));
     CHM_DN chb = AS_CHM_DN(b_M);
     CHM_DN chc = cholmod_allocate_dense(cha->ncol, chb->ncol, cha->ncol,
 					chb->xtype, &c);
@@ -569,22 +623,27 @@ SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j)
     if (csize >= 0 && !isInteger(j))
 	error(_("Index j must be NULL or integer"));
 
+#define CHM_SUB(_M_, _i_, _j_)					\
+    cholmod_submatrix(_M_,					\
+		      (rsize < 0) ? NULL : INTEGER(_i_), rsize,	\
+		      (csize < 0) ? NULL : INTEGER(_j_), csize,	\
+		      TRUE, TRUE, &c)
+    CHM_SP ans;
     if (!chx->stype) {/* non-symmetric Matrix */
-	return chm_sparse_to_SEXP(cholmod_submatrix(chx,
-						    (rsize < 0) ? NULL : INTEGER(i), rsize,
-						    (csize < 0) ? NULL : INTEGER(j), csize,
-						    TRUE, TRUE, &c),
-				  1, 0, Rkind, "",
-				  /* FIXME: drops dimnames */ R_NilValue);
+	ans = CHM_SUB(chx, i, j);
     }
-				/* for now, cholmod_submatrix() only accepts "generalMatrix" */
-    CHM_SP tmp = cholmod_copy(chx, /* stype: */ 0, chx->xtype, &c);
-    CHM_SP ans = cholmod_submatrix(tmp,
-				   (rsize < 0) ? NULL : INTEGER(i), rsize,
-				   (csize < 0) ? NULL : INTEGER(j), csize,
-				   TRUE, TRUE, &c);
-    cholmod_free_sparse(&tmp, &c);
-    return chm_sparse_to_SEXP(ans, 1, 0, Rkind, "", R_NilValue);
+    else {
+	/* for now, cholmod_submatrix() only accepts "generalMatrix" */
+	CHM_SP tmp = cholmod_copy(chx, /* stype: */ 0, chx->xtype, &c);
+	ans = CHM_SUB(tmp, i, j);
+	cholmod_free_sparse(&tmp, &c);
+    }
+
+    // "FIXME": currently dropping dimnames, and adding them afterwards in R :
+    /* // dimnames: */
+    /* SEXP x_dns = GET_SLOT(x, Matrix_DimNamesSym), */
+    /* 	dn = PROTECT(allocVector(VECSXP, 2)); */
+    return chm_sparse_to_SEXP(ans, 1, 0, Rkind, "", /* dimnames: */ R_NilValue);
 }
 
 #define _d_Csp_
@@ -631,40 +690,49 @@ SEXP Csparse_MatrixMarket(SEXP x, SEXP fname)
  *
  * @return  a SEXP, either a (double) number or a length n-vector of diagonal entries
  */
-SEXP diag_tC_ptr(int n, int *x_p, double *x_x, int *perm, SEXP resultKind)
+SEXP diag_tC_ptr(int n, int *x_p, double *x_x, Rboolean is_U, int *perm,
 /*                                ^^^^^^ FIXME[Generalize] to int / ... */
+		 SEXP resultKind)
 {
     const char* res_ch = CHAR(STRING_ELT(resultKind,0));
-    enum diag_kind { diag, diag_backpermuted, trace, prod, sum_log
+    enum diag_kind { diag, diag_backpermuted, trace, prod, sum_log, min, max, range
     } res_kind = ((!strcmp(res_ch, "trace")) ? trace :
 		  ((!strcmp(res_ch, "sumLog")) ? sum_log :
 		   ((!strcmp(res_ch, "prod")) ? prod :
-		    ((!strcmp(res_ch, "diag")) ? diag :
-		     ((!strcmp(res_ch, "diagBack")) ? diag_backpermuted :
-		      -1)))));
-    int i, n_x, i_from = 0;
+		    ((!strcmp(res_ch, "min")) ? min :
+		     ((!strcmp(res_ch, "max")) ? max :
+		      ((!strcmp(res_ch, "range")) ? range :
+		       ((!strcmp(res_ch, "diag")) ? diag :
+			((!strcmp(res_ch, "diagBack")) ? diag_backpermuted :
+			 -1))))))));
+    int i, n_x, i_from;
     SEXP ans = PROTECT(allocVector(REALSXP,
 /*                                 ^^^^  FIXME[Generalize] */
 				   (res_kind == diag ||
-				    res_kind == diag_backpermuted) ? n : 1));
+				    res_kind == diag_backpermuted) ? n :
+				   (res_kind == range ? 2 : 1)));
     double *v = REAL(ans);
 /*  ^^^^^^      ^^^^  FIXME[Generalize] */
 
+    i_from = (is_U ? -1 : 0);
+
 #define for_DIAG(v_ASSIGN)					\
-    for(i = 0; i < n; i++, i_from += n_x) {			\
+    for(i = 0; i < n; i++) {					\
 	/* looking at i-th column */				\
 	n_x = x_p[i+1] - x_p[i];/* #{entries} in this column */	\
+	if( is_U) i_from += n_x;                                \
 	v_ASSIGN;						\
+	if(!is_U) i_from += n_x;                                \
     }
 
     /* NOTA BENE: we assume  -- uplo = "L" i.e. lower triangular matrix
      *            for uplo = "U" (makes sense with a "dtCMatrix" !),
-     *            should use  x_x[i_from + (nx - 1)] instead of x_x[i_from],
-     *            where nx = (x_p[i+1] - x_p[i])
+     *            should use  x_x[i_from + (n_x - 1)] instead of x_x[i_from],
+     *            where n_x = (x_p[i+1] - x_p[i])
      */
 
     switch(res_kind) {
-    case trace:
+    case trace: // = sum
 	v[0] = 0.;
 	for_DIAG(v[0] += x_x[i_from]);
 	break;
@@ -679,6 +747,23 @@ SEXP diag_tC_ptr(int n, int *x_p, double *x_x, int *perm, SEXP resultKind)
 	for_DIAG(v[0] *= x_x[i_from]);
 	break;
 
+    case min:
+	v[0] = R_PosInf;
+	for_DIAG(if(v[0] > x_x[i_from]) v[0] = x_x[i_from]);
+	break;
+
+    case max:
+	v[0] = R_NegInf;
+	for_DIAG(if(v[0] < x_x[i_from]) v[0] = x_x[i_from]);
+	break;
+
+    case range:
+	v[0] = R_PosInf;
+	v[1] = R_NegInf;
+	for_DIAG(if(v[0] > x_x[i_from]) v[0] = x_x[i_from];
+		 if(v[1] < x_x[i_from]) v[1] = x_x[i_from]);
+	break;
+
     case diag:
 	for_DIAG(v[i] = x_x[i_from]);
 	break;
@@ -708,6 +793,7 @@ SEXP diag_tC_ptr(int n, int *x_p, double *x_x, int *perm, SEXP resultKind)
  * Extract the diagonal entries from *triangular* Csparse matrix  __or__ a
  * cholmod_sparse factor (LDL = TRUE).
  *
+ * @param obj -- now a cholmod_sparse factor or a dtCMatrix
  * @param pslot  'p' (column pointer)   slot of Csparse matrix/factor
  * @param xslot  'x' (non-zero entries) slot of Csparse matrix/factor
  * @param perm_slot  'perm' (= permutation vector) slot of corresponding CHMfactor;
@@ -716,17 +802,27 @@ SEXP diag_tC_ptr(int n, int *x_p, double *x_x, int *perm, SEXP resultKind)
  *
  * @return  a SEXP, either a (double) number or a length n-vector of diagonal entries
  */
-SEXP diag_tC(SEXP pslot, SEXP xslot, SEXP perm_slot, SEXP resultKind)
+SEXP diag_tC(SEXP obj, SEXP resultKind)
 {
+
+    SEXP
+	pslot = GET_SLOT(obj, Matrix_pSym),
+	xslot = GET_SLOT(obj, Matrix_xSym);
+    Rboolean is_U = (R_has_slot(obj, Matrix_uploSym) &&
+		     *CHAR(asChar(GET_SLOT(obj, Matrix_uploSym))) == 'U');
     int n = length(pslot) - 1, /* n = ncol(.) = nrow(.) */
-	*x_p  = INTEGER(pslot),
-	*perm = INTEGER(perm_slot);
+	*x_p  = INTEGER(pslot), pp = -1, *perm;
     double *x_x = REAL(xslot);
 /*  ^^^^^^        ^^^^ FIXME[Generalize] to INTEGER(.) / LOGICAL(.) / ... xslot !*/
 
-    return diag_tC_ptr(n, x_p, x_x, perm, resultKind);
+    if(R_has_slot(obj, Matrix_permSym))
+	perm = INTEGER(GET_SLOT(obj, Matrix_permSym));
+    else perm = &pp;
+
+    return diag_tC_ptr(n, x_p, x_x, is_U, perm, resultKind);
 }
 
+
 /**
  * Create a Csparse matrix object from indices and/or pointers.
  *
