@@ -4,7 +4,6 @@ library(Matrix)
 
 source(system.file("test-tools.R", package = "Matrix"))# identical3() etc
 doExtras
-
 ### "sparseQR" : Check consistency of methods
 ##   --------
 data(KNex); mm <- KNex$mm; y <- KNex$y
@@ -124,7 +123,7 @@ a <- matrix(round(10 * runif(90)), 10,9); a[a < 7.5] <- 0
 qD <- chkQR(a) ## using base qr
 qS <- chkQR(A) ## using Matrix "sparse qr" -- "structurally rank deficient!
 validObject(qS)# with the validity now (2012-11-18) -- ok, also for "bad" case
-chk.qr.D.S(qD, qS, y = 10 + 1:nrow(A), force=TRUE)
+chk.qr.D.S(qD, qS, y = 10 + 1:nrow(A), force=TRUE)# 6 warnings: "structurally rank deficient"
 try( ## NOTE: *Both* checks  currently fail here:
     chkQR(A, Qinv.chk=TRUE, QtQ.chk=TRUE)
 )
@@ -134,7 +133,7 @@ oo <- options(Matrix.quiet.qr.R = TRUE, Matrix.verbose = TRUE)
 set.seed(101)
 
 for(N in 1:(if(doExtras) 1008 else 24)) {
-    A <- rSparseMatrix(8,5, nnz = rpois(1, lambda=16))
+    A <- rsparsematrix(8,5, nnz = rpois(1, lambda=16))
     cat(sprintf("%4d -", N))
     checkQR.DS.both(A, Qinv.chk= NA, QtQ.chk=NA)
     ##                          --- => FALSE if struct. rank deficient
@@ -209,7 +208,7 @@ try( checkQR.DS.both(A5, FALSE,  TRUE) )
 
 
 for(N in 1:(if(doExtras) 2^12 else 128)) {
-    A <- round(100*rSparseMatrix(5,3, nnz = min(15,rpois(1, lambda=10))))
+    A <- round(100*rsparsematrix(5,3, nnz = min(15,rpois(1, lambda=10))))
     if(any(apply(A, 2, function(x) all(x == 0)))) ## "column of all 0"
         next
     cat(sprintf("%4d -", N))
@@ -315,22 +314,40 @@ for(n in c(5:12)) {
     ## the inverse permutation:
     invP <- solve(P)@perm
     lDet <- sum(2* log(d))# the "true" value
-    ldet <- Matrix:::.diag.dsC(Chx = CAp, res.kind = "sumLog")
+    ldetp  <-         Matrix:::.diag.dsC(Chx = CAp, res.kind = "sumLog")
+    ldetp. <- sum(log(Matrix:::.diag.dsC(Chx = CAp, res.kind = "diag") ))
     ##
     CA	<- Cholesky(A,perm=FALSE)
-    ldet2 <- Matrix:::.diag.dsC(Chx = CA, res.kind = "sumLog")
+    ldet <- Matrix:::.diag.dsC(Chx = CA, res.kind = "sumLog")
     ## not printing CAp : ends up non-integer for n >= 11
     mCAp <- as(CAp,"sparseMatrix")
     print(mCA  <- drop0(as(CA, "sparseMatrix")))
     stopifnot(identical(A[p,p], as(P %*% A %*% t(P),
 				   "symmetricMatrix")),
-	      all.equal(lDet, sum(log(Matrix:::.diag.dsC(Chx= CAp,res.kind="diag")))),
 	      relErr(d.^2, Matrix:::.diag.dsC(Chx= CA, res.kind="diag")) < 1e-14,
-	      all.equal(lDet, ldet),
-	      all.equal(lDet, ldet2),
 	      relErr(A[p,p], tcrossprod(mCAp)) < 1e-14)
+    if(FALSE)
+        rbind(lDet,ldet, ldetp, ldetp.)
+    ##  ==> Empirically, I see  lDet = ldet  !=  ldetp == ldetp.
+    ## if(rr$rcond.A < ...) warning("condition number of A ..." ## <- TODO
+    cat(1,""); assert.EQ.(lDet,  ldet,   tol = 1e-14)
+    cat(2,""); assert.EQ.(ldetp, ldetp., tol = 1e-14)
+    cat(3,""); assert.EQ.(lDet,  ldetp, tol = n^2* 1e-7)# extreme: have seen 0.0011045 !!
 }## for()
 
+mkCholhash <- function(r.all) {
+    ## r.all %*% (2^(2:0)), but only those that do not have NA / "?" :
+    stopifnot(is.character(rn <- rownames(r.all)),
+              is.matrix(r.all), is.logical(r.all))
+    c.rn <- vapply(rn, function(ch) strsplit(ch, " ")[[1]], character(3))
+    ## Now
+    h1 <- function(i) {
+        ok <- c.rn[,i] != "?"
+        r.all[i, ok] %*% 2^((2:0)[ok])
+    }
+    vapply(seq_len(nrow(r.all)), h1, numeric(1))
+}
+
 set.seed(17)
 (rr <- mkLDL(4))
 (CA <- Cholesky(rr$A))
@@ -339,7 +356,7 @@ stopifnot(all.equal(determinant(rr$A) -> detA,
           is.all.equal3(c(detA$modulus), log(det(rr$D)), sum(log(rr$D@x))))
 A12 <- mkLDL(12, 1/10)
 (r12 <- allCholesky(A12$A))[-1]
-aCh.hash <- r12$r.all %*% (2^(2:0))
+aCh.hash <- mkCholhash(r12$r.all)
 if(FALSE)## if(require("sfsmisc"))
 split(rownames(r12$r.all), Duplicated(aCh.hash))
 
@@ -351,7 +368,7 @@ for(n in 1:50) { ## used to seg.fault at n = 10 !
     ## Compare .. apart from the NAs that happen from (perm=FALSE, super=TRUE)
     iNA <- apply(is.na(r$r.all), 1, any)
     cat(sprintf(" -> %3s NAs\n", if(any(iNA)) format(sum(iNA)) else "no"))
-    stopifnot(aCh.hash[!iNA] == r$r.all[!iNA,] %*% (2^(2:0)))
+    stopifnot(aCh.hash[!iNA] == mkCholhash(r$r.all[!iNA,]))
 ##     cat("--------\n")
 }
 
@@ -390,7 +407,7 @@ CAinv2 <- solve(CA, Diagonal(nrow(A.)))
 CAinv2 <- as(CAinv2, "symmetricMatrix")
 stopifnot(identical(CAinv, CAinv2))
 
-## FINALLY fix this "TODO":
+## FINALLY fix "TODO": (not implemented *symbolic* factorization of nMatrix)
 try(    tc <- Cholesky(nT.)  )
 
 for(p in c(FALSE,TRUE))
